@@ -133,10 +133,7 @@ def should_skip(core):
         only_one = os.getenv("COVERAGE_ONE_CORE")
         if only_one:
             if CPYTHON:
-                if sys.version_info >= (3, 12):
-                    if core != "sysmon":
-                        skipper = f"Only one core: not running {core}"
-                elif core != "ctrace":
+                if core != "ctrace":
                     skipper = f"Only one core: not running {core}"
             else:
                 if core != "pytrace":
