@@ -5,12 +5,14 @@
 
 from __future__ import annotations
 
+import re
 import textwrap
+from unittest import mock
 
 import pytest
 
 from coverage import env
-from coverage.exceptions import NotPython
+from coverage.exceptions import NoSource, NotPython
 from coverage.parser import PythonParser
 
 from tests.coveragetest import CoverageTest
@@ -151,6 +153,24 @@ class PythonParserTest(PythonParserTestBase):
         assert expected_arcs == parser.arcs()
         assert expected_exits == parser.exit_counts()
 
+    def test_nested_context_managers(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/1876
+        parser = self.parse_text("""\
+            a = 1
+            with suppress(ValueError):
+                with suppress(ValueError):
+                    x = 4
+                with suppress(ValueError):
+                    x = 6
+                with suppress(ValueError):
+                    x = 8
+            a = 9
+            """)
+
+        one_nine = set(range(1, 10))
+        assert parser.statements == one_nine
+        assert parser.exit_counts() == dict.fromkeys(one_nine, 1)
+
     def test_module_docstrings(self) -> None:
         parser = self.parse_text("""\
             '''The docstring on line 1'''
@@ -177,6 +197,24 @@ class PythonParserTest(PythonParserTestBase):
         with pytest.raises(NotPython, match=msg):
             self.parse_text("]")
 
+    def test_bug_1891(self) -> None:
+        # These examples exercise code paths I thought were impossible.
+        parser = self.parse_text("""\
+            res = siblings(
+                'configure',
+                **ca,
+            )
+            """)
+        assert parser.exit_counts() == {1: 1}
+        parser = self.parse_text("""\
+            def g2():
+                try:
+                    return 2
+                finally:
+                    return 3
+            """)
+        assert parser.exit_counts() == {1: 1, 2: 1, 3: 1, 5: 1}
+
 
 class ExclusionParserTest(PythonParserTestBase):
     """Tests for the exclusion code in PythonParser."""
@@ -655,6 +693,19 @@ class ExclusionParserTest(PythonParserTestBase):
         )
         assert parser.statements == {2, 4}
 
+    def test_no_exclude_at_all(self) -> None:
+        parser = self.parse_text("""\
+            def foo():
+                if fooey:
+                    a = 3
+                else:
+                    a = 5
+            b = 6
+            """,
+            exclude="",
+        )
+        assert parser.exit_counts() == { 1:1, 2:2, 3:1, 5:1, 6:1 }
+
     def test_formfeed(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/461
         parser = self.parse_text("""\
@@ -1116,3 +1167,10 @@ class ParserFileTest(CoverageTest):
 
         parser = self.parse_file("abrupt.py")
         assert parser.statements == {1}
+
+    def test_os_error(self) -> None:
+        self.make_file("cant-read.py", "BOOM!")
+        msg = "No source for code: 'cant-read.py': Fake!"
+        with pytest.raises(NoSource, match=re.escape(msg)):
+            with mock.patch("coverage.python.read_python_source", side_effect=OSError("Fake!")):
+                PythonParser(filename="cant-read.py")
